<?php

namespace App\Entity;

use App\Repository\UserRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Security\Core\User\LegacyPasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Validator\Constraints as Assert;

#[ORM\Entity(repositoryClass:UserRepository::class)]
#[ORM\Table(name:'`user`')]
#[UniqueEntity(fields:['email'], message:'email.unique')]
class User implements UserInterface, LegacyPasswordAuthenticatedUserInterface
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column(type:'integer')]
    private int $id;

    #[ORM\Column(name: 'accept_gnl_conditions', type: 'boolean')]
    private bool $acceptGeneralConditions;

    #[ORM\Column(name: 'inscription_newsletter', type: 'boolean')]
    private bool $has_subscribed_newsletter;

    #[Assert\Email(message: 'email.valid')]
    #[ORM\Column(type:'string', length:255)]
    private string $email;

    #[ORM\Column(type:'string', length:255, unique:true)]
    private string $email_canonical;

    #[ORM\Column(name: 'name', type: 'string')]
    private string $lastName;

    #[ORM\Column(name: 'firstname', type: 'string')]
    private string $firstName;

    #[ORM\Column(type:'string')]
    private string $username;

    #[ORM\Column(name: 'username_canonical', type: 'string', unique: true)]
    private string $username_canonical;

    #[ORM\Column(name: 'credential_expired', type: 'boolean')]
    private bool $credentialExpired = false;

    #[ORM\Column(type:'json')]
    private array $roles = []; /** @phpstan-ignore-line */

    /**
     * hashed password
     */
    #[ORM\Column(type:'string')]
    private string $password;

    #[ORM\Column(name: 'enabled', type: 'boolean')]
    private bool $isVerified;

    #[ORM\Column(name: 'salt', type: 'string', length: 255)]
    private string $salt;

    /**
     * @var Collection<Capsule>
     */
    #[ORM\ManyToMany(targetEntity:'App\Entity\Capsule', mappedBy:'editors')]
    private Collection $capsules;

    /**
     * @var Collection<Group>
     */
    #[ORM\OneToMany(mappedBy: 'author', targetEntity: 'App\Entity\Group')]
    private Collection $groups;

    /**
     * plain password to store before hashing it
     */
    public string $plainPassword;

    #[ORM\Column(type:'string')]
    public string $locale;

    public function __construct()
    {
        $this->capsules = new ArrayCollection();
        $this->groups = new ArrayCollection();
        $this->acceptGeneralConditions = false;
        $this->isVerified = false;
        $this->credentialExpired = false;
        $this->has_subscribed_newsletter = false;
    }

    public function getId(): int
    {
        return $this->id;
    }

    public function getEmail(): string
    {
        return $this->email;
    }

    public function setEmail(string $email): self
    {
        $this->email = $email;
        $this->email_canonical = strtolower($email);
        $this->username = $email;
        $this->username_canonical = strtolower($email);
        return $this;
    }

    /**
     * A visual identifier that represents this user.
     *
     * @see UserInterface
     */
    public function getUserIdentifier(): string
    {
        return $this->email;
    }

    /**
     * @deprecated since Symfony 5.3, use getUserIdentifier instead
     */
    public function getUsername(): string
    {
        return $this->email;
    }

    /**
     * @see UserInterface
     */
    public function getRoles(): array
    {
        $roles = $this->roles;
        // guarantee every user at least has ROLE_USER
        $roles[] = 'ROLE_USER';

        return array_unique($roles);
    }

    /**
     * @param array<string> $roles
     */
    public function setRoles(array $roles): self
    {
        $this->roles = $roles;

        return $this;
    }

    /**
     * @see PasswordAuthenticatedUserInterface
     */
    public function getPassword(): string
    {
        return $this->password;
    }

    public function setPassword(string $password): self
    {
        $this->password = $password;

        return $this;
    }

    /**
     * Returning a salt is only needed, if you are not using a modern
     * hashing algorithm (e.g. bcrypt or sodium) in your security.yaml.
     *
     * @see UserInterface
     */
    public function getSalt(): string
    {
        return $this->salt;
    }

    /**
     * @see UserInterface
     */
    public function eraseCredentials(): void
    {
        // If you store any temporary, sensitive data on the user, clear it here
        $this->plainPassword = "";
    }

    public function isVerified(): bool
    {
        return $this->isVerified;
    }

    public function setIsVerified(bool $isVerified): self
    {
        $this->isVerified = $isVerified;

        return $this;
    }

    public function getFirstName(): string
    {
        return $this->firstName;
    }

    public function getLastName(): ?string
    {
        return $this->lastName;
    }

    public function setFirstName(string $firstName): void
    {
        $this->firstName = $firstName;
    }

    public function setLastName(string $name): void
    {
        $this->lastName = $name;
    }

    public function setSalt(string $salt): void
    {
        $this->salt = $salt;
    }

    public function getFullName(): string
    {
        return $this->firstName . " " . $this->lastName;
    }

    /**
     * @return Collection<Capsule>
     */
    public function getCapsules(): Collection
    {
        return $this->capsules;
    }

    public function addCapsule(Capsule $capsule): void
    {
        $this->capsules[] = $capsule;
    }

    public function removeCapsule(Capsule $capsule): User
    {
        $this->capsules->removeElement($capsule);
        return $this;
    }

    public function hasAcceptGeneralConditions(): bool
    {
        return $this->acceptGeneralConditions;
    }

    public function setAcceptGeneralConditions(bool $acceptGeneralConditions): void
    {
        $this->acceptGeneralConditions = $acceptGeneralConditions;
    }

    public function hasSubscribedToNewsletter(): bool
    {
        return $this->has_subscribed_newsletter;
    }

    public function setSubscribedToNewsLetter(bool $is_subscribed_news_letter): void
    {
        $this->has_subscribed_newsletter = $is_subscribed_news_letter;
    }

    public function getLocale(): string
    {
        return $this->locale;
    }

    public function setLocale(string $locale): void
    {
        $this->locale = $locale;
    }

    /**
     * @return Collection<Group>
     */
    public function getGroups(): Collection
    {
        return $this->groups;
    }

    public function setGroupAll(Group $group_all): void
    {
        $groups = $this->getGroups()->toArray();
        array_unshift($groups, $group_all);
    }

    public function addGroup(Group $group): void
    {
        $this->groups[] = $group;
    }

    public function removeGroup(Group $group): User
    {
        $this->groups->removeElement($group);
        return $this;
    }

    /**
     * @param Capsule $capsule
     * @return array<Group>
     */
    public function getGroupsByCapsule(Capsule $capsule): array
    {
        return array_uintersect(
            $this->getGroups()->toArray(),
            $capsule->getGroups()->toArray(),
            function ($obj1, $obj2) {
                return $obj1->getId() - $obj2->getId();
            }
        );
    }
}
